###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.7 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import Products
import base_item
import mail_gateway
import mailbox
import cStringIO
import message_container
import news_group_folder
import message_filter_folder

from Products import RFC822Message

gum_add_redirect = HTMLFile('gum_add_redirect', globals())
manage_add_gum_form = HTMLFile('gum_add', globals())


def manage_add_gum(self, id, local_username, title, REQUEST=None):
    "Add an instance of the gum class"

    self._setObject(id, gum(id, local_username, ''))

    if REQUEST is not None:

        return self.manage_main(self, REQUEST)


class gum(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	counter = 0

	meta_type = 'GUM'
	icon = 'misc_/GUM/Folder_icon.gif'
	#default_catalog = 'Catalog'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'local_username', 'type':'string'},
		)

	meta_types = (
		{'name':'MailGateway',
		 'action':'manage_add_mail_gateway_form'},
		{'name':'MessageContainer',
		 'action':'manage_add_message_container_form'},
		{'name':'NewsGroupFolder',
		 'action':'manage_add_news_group_folder_form'},
		{'name':'MessageFilterFolder',
		 'action':'manage_add_message_filter_folder_form'}
		)

	manage_add_mail_gateway_form = mail_gateway.manage_add_mail_gateway_form
	manage_add_mail_gateway = mail_gateway.manage_add_mail_gateway
	mail_gateway_add_redirect = mail_gateway.add_mail_gateway_redirect

	manage_add_message_container_form = message_container.manage_add_message_container_form
	manage_add_message_container = message_container.manage_add_message_container
	message_container_add_redirect = message_container.message_container_add_redirect

	manage_add_news_group_folder_form = news_group_folder.manage_add_news_group_folder_form
	manage_add_news_group_folder = news_group_folder.manage_add_news_group_folder
	news_group_folder_add_redirect = news_group_folder.news_group_folder_add_redirect

	manage_add_message_filter_folder_form = message_filter_folder.manage_add_message_filter_folder_form
	manage_add_message_filter_folder = message_filter_folder.manage_add_message_filter_folder
	message_filter_folder_add_redirect = message_filter_folder.message_filter_folder_add_redirect	

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add MailGateway',
		 ('manage_add_mail_gateway', 'manage_add_mail_gateway_form')),
		('Add MessageContainer',
		 ('manage_add_message_container', 'manage_add_message_container_form')),
		('View GUMs',
		 ('',)),
		)

	index_html = HTMLFile('gum_index', globals())

	product_name = 'GUM'


	def __init__(self, id, local_username, title='', primary_folder='Inbox'):
		"Initialize the class instance"

		# Add Inbox and Outgoing

		message_container.manage_add_message_container(self, 'Inbox', 'Inbox')
		message_container.manage_add_message_container(self, 'Outgoing', 'Outgoing')		
		message_container.manage_add_message_container(self, 'Temporary', 'Temporary')

		message_filter_folder.manage_add_message_filter_folder(self, 'MessageFilters', 'MessageFilters')

		self.id = id
		self.title = title

		self.local_username = local_username

		self.primary_folder = primary_folder
		
		self.index_object()


	def get_unique_id(self):
		"""Returns a unique identifier."""

		try:

			return self.getParentNode().get_id()

		except AttributeError:

			self.counter = self.counter + 1

			return str(self.counter)


	def get_temporary_folder(self):
		"""Returns the temporary folder."""

		return self['Temporary']


	def get_messages(self, REQUEST=None, RESPONSE=None):
		"""Retrieves messages from the local server."""

		message_filters = []

		for filter_folder in self.objectValues('MessageFilterFolder'):

			for message_filter in filter_folder.objectValues('MessageFilter'):

				message_filters.append(message_filter)

		primary_folder = self[self.primary_folder]

		mbox = mailbox.UnixMailbox(
			open('/var/spool/mail/'+self.local_username))

		#mbox = mailbox.UnixMailbox(
		#	open('/tmp/mailbox'))

		while 1:

			_message = mbox.next()

			if not _message:

				break

			_message.fp.seek(0)
			_message = RFC822Message.util.my_string_io(_message.fp.read())

			message = RFC822Message.rfc822_message_parse.rfc822_message_parse(_message)

			for message_filter in message_filters:

				if message.headers.has_key(message_filter.header_name):

					if message_filter._regexp_rule.search(message.headers[message_filter.header_name]):

						RFC822Message.util.store_message(
							self[message_filter.message_container],
							message, readonly=1)

						break

				else:

					# Couldn't match with regexp
					# Remove this else clause FIXME

					pass

			else:

				RFC822Message.util.store_message(
					primary_folder, message, readonly=1)


		return 'success'

default__class_init__(gum)
