###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.11 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import rfc822_attachment
import rfc822_address_container
import rfc822_address
import base_item
import mimetools
import string

add_rfc822_message_redirect = HTMLFile('rfc822_message_add_redirect', globals())
manage_add_rfc822_message_form = HTMLFile('rfc822_message_add', globals())


#def manage_add_rfc822_message(self, id, to=[], sender='', reply_to='', cc=[], bcc=[],
#							  newsgroups=[], body='', subject='', headers={},
#							  REQUEST=None):
#   self._setObject(id, rfc822_message(id, to, sender, reply_to, cc,
#									   bcc, newsgroups, body, subject,
#									   headers))
#
#    if REQUEST is not None:
#
#        return self.manage_main(self, REQUEST)


def manage_add_rfc822_message(self, id, sender='', reply_to='', body='', subject='', REQUEST=None):
    "Add an instance of the rfc822_message class"

    self._setObject(id, rfc822_message(sender, reply_to, body, subject))

    if REQUEST is not None:

        return self.manage_main(self, REQUEST)


class rfc822_message(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'RFC822Message'
	icon = 'misc_/RFC822Message/Folder_icon.gif'

	_properties = (
		{'id':'sender', 'type':'string'},
		{'id':'reply_to', 'type':'string'},
		{'id':'newsgroups', 'type':'lines'},
		{'id':'body', 'type':'text'},
		{'id':'readonly', 'type':'boolean'},
		)

	meta_types = (
		{'name':'RFC822Attachment', 'action':'manage_add_rfc822_attachment_form'},
		{'name':'RFC822Address', 'action':'manage_add_rfc822_address_form'},
		{'name':'RFC822AddressContainer', 'action':'manage_add_rfc822_address_container_form'},
		)

	manage_add_rfc822_attachment_form = rfc822_attachment.manage_add_rfc822_attachment_form
	manage_add_rfc822_attachment = rfc822_attachment.manage_add_rfc822_attachment
	rfc822_attachment_add_redirect = rfc822_attachment.rfc822_attachment_add_redirect

	manage_add_rfc822_address_container_form = rfc822_address_container.manage_add_rfc822_address_container_form
	manage_add_rfc822_address_container = rfc822_address_container.manage_add_rfc822_address_container
	rfc822_address_container_add_redirect = rfc822_address_container.rfc822_address_container_add_redirect

	#manage_add_rfc822_address_form = rfc822_address.manage_add_rfc822_address_form
	#manage_add_rfc822_address = rfc822_address.manage_add_rfc822_address
	#rfc822_address_add_redirect = rfc822_address.rfc822_address_add_redirect	

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add RFC822Attachment',
		 ('manage_add_rfc822_attachment', 'manage_add_rfc822_attachment_form')),
		('Add RFC822Address',
		 ('manage_add_rfc822_address', 'manage_add_rfc822_address_form')),
		('View RFC822Messages',
		 ('',)),
		)

	index_html = HTMLFile('rfc822_message_index', globals())

	view_index_html = HTMLFile('rfc822_message_view_index', globals())
	edit_index_html = HTMLFile('rfc822_message_edit_index', globals())


	def copy(self):
		"""Copies message to temporary folder."""

		temporary = self.get_temporary_folder()

		message_copy = self.getParentNode().manage_copyObjects(self.id)
		temporary.manage_pasteObjects(cb_copy_data=message_copy)

		# We do this so we never get id conflicts.

		new_id = str(self.get_unique_id())

		temporary.manage_renameObject(self.id, new_id)

		message_new = temporary[new_id]

		return message_new


	def delete_address_fields(self):
		"""Removes all recipients."""

		for address_field in ['to', 'cc', 'bcc']:

			for address in self[address_field].objectValues():

				self[address_field].manage_delObject(address.id)


	def forward(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for forwarding."""

		message_new = self.copy()

		message_new.readonly = 0
		message_new.subject = self.render_forward_subject()

		message_new.delete_address_fields()

		RESPONSE.redirect(message_new.absolute_url())



	def reply(self, REQUEST=None, RESPONSE=None, all=None):
		"""Method that prepares a message for replying."""

		new_id = str(self.get_unique_id())

		new_body = '> ' + string.join(string.split(self.body, '\n> '))

		if self.headers.has_key('references'):

			references = string.split(self.headers['references'])

		else:

			references = []

		headers = {}

		references.append('<' + self.headers['message-id'] + '>')

		headers['references'] = string.join(references, ' ')

		temporary_folder = self.get_temporary_folder()

		if all is not None:

			cc = self.cc
			bcc = self.bcc

		else:

			cc = []
			bcc = []

		temporary_folder.manage_add_rfc822_message(new_id, to=self.to,
												   sender=self.mail_address,
												   cc=cc, bcc=bcc,
												   newsgroups=self.newsgroups,
												   body=new_body,
												   subject=self.render.reply_subject(),
												   headers=headers
												   )


	def reply_all(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for replying."""

		self.reply(REQUEST=REQUEST, RESPONSE=RESPONSE, all=1)


	def render_reply_subject(self):
		"""Method for rendering an appropriate subject upon replies."""

		subject = self.subject

		if string.lower(subject)[:3] == 're:':

			return subject

		else:

			return 'Re: ' + subject


	def render_forward_subject(self):
		"""Method for rendering an appropriate subject upon forwards."""

		subject = self.subject

		if string.lower(subject)[:4] == 'fwd:':

			return subject

		else:

			return 'Fwd: ' + subject


	def get_filled_address_fields(self):
		"""Returns address fields that contain one or more addresses."""

		address_fields = []

		for address_field in ['to', 'cc', 'bcc']:

			if len(self[address_field].objectValues('RFC822Address')):

				address_fields.append(address_field)

		return address_fields


	def render_address_field(self, field='to'):
		"""Renders the addresses contained in the given field."""

		return_string = ""
		
		for object in self[field].objectValues():

			return_string = return_string + '%s, ' % object.address

		else:

			return_string = return_string[:-2]

		return return_string


	def add_recipient(address_field='to', address='', realname='', comment=''):
		"""Adds a recipient to the given address field."""

		new_id = str(self.get_unique_id())

		self[address_field].manage_addDTMLDocument(new_id, '')

		address_block = self.address_field[new_id]

		address_block.address = address
		address_block.comment = comment
		address_block.realname = realname

		#return address_block


	def add_newsgroup(self, newsgroup):
		"""Adds a newsgroup to the message."""

		try:

			self.newsgroups.index(newsgroup)
			return

		except:

			self.newsgroups.append(newsgroup)


	def __init__(self, id, sender, body, subject, reply_to=''):
		"Initialize the class instance"

		self.id = id

		self.sender = sender
		self.reply_to = reply_to
		self.body = body
		self.subject = subject

		self.newsgroups = []
		self.headers = {}

		self.index_object()

		self.manage_add_rfc822_address_container('to', title='to')
		self.manage_add_rfc822_address_container('cc', title='cc')
		self.manage_add_rfc822_address_container('bcc', title='bcc')


default__class_init__(rfc822_message)
